@@ -41,8 +41,6 @@ class Query():
 
     music_folder = None
     MUSIC_URI = None
-    download_folder = None
-    DOWNLOAD_URI = None
 
     @log
     def __init__(self):
@@ -53,21 +51,13 @@ class Query():
             logger.warn("XDG Music dir is not set")
             return
 
-        try:
-            Query.download_folder = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_DOWNLOAD)
-            assert Query.download_folder is not None
-        except (TypeError, AssertionError):
-            logger.warn("XDG Download dir is not set")
-            return
-
         Query.MUSIC_URI = Tracker.sparql_escape_string(GLib.filename_to_uri(Query.music_folder))
-        Query.DOWNLOAD_URI = Tracker.sparql_escape_string(GLib.filename_to_uri(Query.download_folder))
 
-        for folder in [Query.music_folder, Query.download_folder]:
+        for folder in [Query.music_folder]:
             if os.path.islink(folder):
                 logger.warn("%s is a symlink, this folder will be omitted", folder)
             else:
-                i = len(next(os.walk(folder))[2])
+                i = len(next(os.walk(folder)))
                 logger.debug("Found %d files in %s", i, folder)
 
     def __repr__(self):
@@ -84,8 +74,8 @@ class Query():
         # list should include 'the' regardless of language. If some articles occur more
         # frequently than others, most common should appear first, least common last.
         for article in reversed(_("the a an").split(" ")):
-            return_statement = '''IF(fn:starts-with(fn:lower-case(%(attribute)s), "%(article)s"),
-            fn:substring(fn:lower-case(%(attribute)s), %(substr_start)s),
+            return_statement = '''IF(STRSTARTS(%(attribute)s, "%(article)s"),
+            SUBSTR(%(attribute)s, %(substr_start)s),
             %(nested_if)s)''' % {
                 'attribute': attr,
                 'article': article + " ",
@@ -114,29 +104,14 @@ class Query():
         query = '''
     SELECT
         COUNT(?song) AS childcount
-    WHERE {
+    {
         ?song a nmm:MusicPiece ;
-              a nfo:FileDataObject
-        FILTER (
-            tracker:uri-is-descendant(
-                '%(music_dir)s', nie:url(?song)
-            ) ||
-            tracker:uri-is-descendant(
-                '%(download_dir)s', nie:url(?song)
-            )
-        )
-        FILTER (
-            NOT EXISTS {
-                ?song a nmm:Video
-            } &&
-            NOT EXISTS {
-                ?song a nmm:Playlist
-            }
-        )
+              a nfo:FileDataObject ;
+	      nie:url ?url .
+        FILTER(STRSTARTS(?url, '%(music_dir)s/'))
     }
     '''.replace('\n', ' ').strip() % {
-            'music_dir': Query.MUSIC_URI,
-            'download_dir': Query.DOWNLOAD_URI
+            'music_dir': Query.MUSIC_URI
         }
 
         return query
@@ -144,136 +119,35 @@ class Query():
     @staticmethod
     def albums(where_clause):
         query = '''
-    SELECT DISTINCT
+    SELECT
         rdf:type(?album)
-        tracker:id(?album) AS id
-        (
-            SELECT
-                nmm:artistName(?artist)
-            WHERE {
-                ?album nmm:albumArtist ?artist
-            }
-            LIMIT 1
-        ) AS artist
-        nie:title(?album) AS title
-        nie:title(?album) AS album
-        tracker:coalesce(
-            (
-                SELECT
-                    GROUP_CONCAT(
-                        nmm:artistName(?artist),
-                        ','
-                    )
-                WHERE {
-                    ?album nmm:albumArtist ?artist
-                }
-            ),
-            (
-                SELECT
-                    GROUP_CONCAT(
-                        (
-                            SELECT
-                                nmm:artistName(nmm:performer(?_12)) AS perf
-                            WHERE {
-                                ?_12 nmm:musicAlbum ?album
-                            }
-                            GROUP BY ?perf
-                        ),
-                        ','
-                    ) AS album_performer
-                WHERE {
-                }
-            )
-        ) AS author
-        xsd:integer(
-            tracker:coalesce(
-                nmm:albumTrackCount(?album),
-                (
-                    SELECT
-                        COUNT(?_1)
-                    WHERE {
-                        ?_1 nmm:musicAlbum ?album ;
-                            tracker:available 'true'
-                        FILTER (
-                            tracker:uri-is-descendant(
-                                '%(music_dir)s', nie:url(?_1)
-                            ) ||
-                            tracker:uri-is-descendant(
-                                '%(download_dir)s', nie:url(?_1)
-                            )
-                        )
-                        FILTER (
-                            NOT EXISTS {
-                                ?_1 a nmm:Video
-                            } &&
-                            NOT EXISTS {
-                                ?_1 a nmm:Playlist
-                            }
-                        )
-                    }
-                )
-            )
-        ) AS childcount
-        (
-            SELECT
-                fn:year-from-dateTime(?c)
-            WHERE {
-                ?_2 nmm:musicAlbum ?album ;
-                    nie:contentCreated ?c ;
-                    tracker:available 'true'
-                FILTER (
-                    tracker:uri-is-descendant(
-                        '%(music_dir)s', nie:url(?_2)
-                    ) ||
-                    tracker:uri-is-descendant(
-                        '%(download_dir)s', nie:url(?_2)
-                    )
-                )
-                FILTER (
-                    NOT EXISTS {
-                        ?_2 a nmm:Video
-                    } &&
-                    NOT EXISTS {
-                        ?_2 a nmm:Playlist
-                    }
-                )
-            }
-            LIMIT 1
-        ) AS creation-date
-        {
-            %(where_clause)s
-            FILTER (
-                EXISTS {
-                    ?_3 nmm:musicAlbum ?album ;
-                        tracker:available 'true'
-                    FILTER (
-                        tracker:uri-is-descendant(
-                            '%(music_dir)s', nie:url(?_3)
-                        ) ||
-                        tracker:uri-is-descendant(
-                            '%(download_dir)s', nie:url(?_3)
-                        )
-                    )
-                    FILTER (
-                        NOT EXISTS {
-                            ?_3 a nmm:Video
-                        } &&
-                        NOT EXISTS {
-                            ?_3 a nmm:Playlist
-                        }
-                    )
-                }
-            )
-        }
-    ORDER BY %(album_order)s
-        %(artist_order)s
-        ?albumyear
+        tracker:id(?album) AS ?id
+        nmm:artistName(?albumArtist) AS ?artist
+        ?title
+        tracker:coalesce((SELECT GROUP_CONCAT(nmm:artistName(?albumArtist), ',') { ?album nmm:albumArtist ?albumArtist }),
+                         (SELECT GROUP_CONCAT(nmm:artistName(?performer), ',') { ?song nmm:performer ?performer })) AS ?performer
+        COUNT(?song) AS ?childcount
+        YEAR(MAX(nie:contentCreated(?song))) AS ?creation_date
+    {
+        %(where_clause)s
+        ?song a nmm:MusicPiece ;
+            nmm:musicAlbum ?album ;
+            nmm:performer ?performer .
+        ?album nmm:albumArtist ?albumArtist ;
+            nie:title ?title .
+        BIND(LCASE(?title) AS ?title_lower) .
+        BIND(LCASE(nmm:artistName(?albumArtist)) AS ?artist_lower) .
+        BIND((%(album_order)s) AS ?album_collation) .
+        BIND((%(artist_order)s) AS ?artist_collation) .
+        FILTER(STRSTARTS(nie:url(?song), '%(music_dir)s/'))
+    }
+    GROUP BY ?album
+    ORDER BY ?album_collation ?artist_collation ?creation_date
     '''.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
             'music_dir': Query.MUSIC_URI,
-            'download_dir': Query.DOWNLOAD_URI,
-            'album_order': Query.order_by_statement("?title"),
-            'artist_order': Query.order_by_statement("?author")
+            'album_order': Query.order_by_statement("?title_lower"),
+            'artist_order': Query.order_by_statement("?artist_lower"),
         }
 
         return query
@@ -281,152 +155,38 @@ class Query():
     @staticmethod
     def artists(where_clause):
         query = '''
-    SELECT DISTINCT
+    SELECT
         rdf:type(?album)
         tracker:id(?album) AS id
-        (
-            SELECT
-                nmm:artistName(?artist)
-            WHERE {
-                ?album nmm:albumArtist ?artist
-            }
-            LIMIT 1
-        ) AS artist
-        nie:title(?album) AS title
-        nie:title(?album) AS album
-        tracker:coalesce(
-            (
-                SELECT
-                    GROUP_CONCAT(
-                        nmm:artistName(?artist),
-                        ','
-                    )
-                WHERE {
-                    ?album nmm:albumArtist ?artist
-                }
-            ),
-            (
-                SELECT
-                    GROUP_CONCAT(
-                        (
-                            SELECT
-                                nmm:artistName(nmm:performer(?_12)) AS perf
-                            WHERE {
-                                ?_12 nmm:musicAlbum ?album
-                                FILTER (
-                                    tracker:uri-is-descendant(
-                                        '%(music_dir)s', nie:url(?_12)
-                                    ) ||
-                                    tracker:uri-is-descendant(
-                                        '%(download_dir)s', nie:url(?_12)
-                                    )
-                                )
-                                FILTER (
-                                    NOT EXISTS {
-                                        ?_12 a nmm:Video
-                                    } &&
-                                    NOT EXISTS {
-                                        ?_12 a nmm:Playlist
-                                    }
-                                )
-                            }
-                            GROUP BY ?perf
-                        ),
-                        ','
-                    ) AS album_performer
-                WHERE {
-                }
-            )
-        ) AS author
-        xsd:integer(
-            tracker:coalesce(
-                nmm:albumTrackCount(?album),
-                (
-                    SELECT
-                        COUNT(?_1)
-                    WHERE {
-                        ?_1 nmm:musicAlbum ?album ;
-                        tracker:available 'true'
-                        FILTER (
-                            tracker:uri-is-descendant(
-                                '%(music_dir)s', nie:url(?_1)
-                            ) ||
-                            tracker:uri-is-descendant(
-                                '%(download_dir)s', nie:url(?_1)
-                            )
-                        )
-                        FILTER (
-                            NOT EXISTS {
-                                ?_1 a nmm:Video
-                            } &&
-                            NOT EXISTS {
-                                ?_1 a nmm:Playlist
-                            }
-                        )
-                    }
-                )
-            )
-        ) AS childcount
-        (
-            SELECT
-                fn:year-from-dateTime(?c)
-            WHERE {
-                ?_2 nmm:musicAlbum ?album ;
-                    nie:contentCreated ?c ;
-                    tracker:available 'true'
-                FILTER (
-                    tracker:uri-is-descendant(
-                        '%(music_dir)s', nie:url(?_2)
-                    ) ||
-                    tracker:uri-is-descendant(
-                        '%(download_dir)s', nie:url(?_2)
-                    )
-                )
-                FILTER (
-                    NOT EXISTS {
-                        ?_2 a nmm:Video
-                    } &&
-                    NOT EXISTS {
-                        ?_2 a nmm:Playlist
-                    }
-                )
-            }
-            LIMIT 1
-        ) AS creation-date
-        {
-            %(where_clause)s
-            FILTER (
-                EXISTS {
-                    ?_3 nmm:musicAlbum ?album ;
-                        tracker:available 'true'
-                    FILTER (
-                        tracker:uri-is-descendant(
-                            '%(music_dir)s', nie:url(?_3)
-                        ) ||
-                        tracker:uri-is-descendant(
-                            '%(download_dir)s', nie:url(?_3)
-                        )
-                    )
-                    FILTER (
-                        NOT EXISTS {
-                            ?_3 a nmm:Video
-                        } &&
-                        NOT EXISTS {
-                            ?_3 a nmm:Playlist
-                        }
-                    )
-                }
-            )
-        }
-    ORDER BY %(artist_order)s
-        ?albumyear
-        %(album_order)s
+        ?author
+        ?title
+        tracker:coalesce((SELECT GROUP_CONCAT(nmm:artistName(?albumArtist), ',') { ?album nmm:albumArtist ?albumArtist }),
+                         (SELECT GROUP_CONCAT(nmm:artistName(?performer), ',') { ?song nmm:performer ?performer })) AS ?performer
+        COUNT(?song) AS ?childcount
+        YEAR(MAX(nie:contentCreated(?song))) AS ?creation_date
+    {
+        %(where_clause)s
+        ?album a nmm:MusicAlbum ;
+               nmm:albumArtist ?albumArtist ;
+               nie:title ?title .
+        ?albumArtist nmm:artistName ?author .
+        ?song nmm:musicAlbum ?album ;
+              nmm:performer ?performer .
+
+        BIND(LCASE(?author) AS ?author_lower) .
+        BIND(LCASE(?title) AS ?title_lower) .
+        BIND((%(artist_order)s) AS ?artist_collation) .
+        BIND((%(album_order)s) AS ?title_collation) .
+
+        FILTER(STRSTARTS(nie:url(?song), '%(music_dir)s/'))
+    }
+    GROUP BY ?album
+    ORDER BY ?artist_collation ?creation_date ?album_collation
     '''.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
             'music_dir': Query.MUSIC_URI,
-            'download_dir': Query.DOWNLOAD_URI,
-            'artist_order': Query.order_by_statement("?author"),
-            'album_order': Query.order_by_statement("nie:title(?album)")
+            'artist_order': Query.order_by_statement("?author_lower"),
+            'album_order': Query.order_by_statement("?title_lower")
         }
 
         return query
@@ -434,43 +194,32 @@ class Query():
     @staticmethod
     def songs(where_clause):
         query = '''
-    SELECT DISTINCT
+    SELECT
         rdf:type(?song)
-        tracker:id(?song) AS id
-        nie:url(?song) AS url
-        nie:title(?song) AS title
-        nmm:artistName(nmm:performer(?song)) AS artist
-        nie:title(nmm:musicAlbum(?song)) AS album
-        nfo:duration(?song) AS duration
-        IF(bound(?tag), 'truth!', '') AS lyrics
-        {
-            %(where_clause)s
-            OPTIONAL {
-                ?song nao:hasTag ?tag .
-                FILTER( ?tag = nao:predefined-tag-favorite )
-            }
-            FILTER (
-                tracker:uri-is-descendant(
-                    '%(music_dir)s', nie:url(?song)
-                ) ||
-                tracker:uri-is-descendant(
-                    '%(download_dir)s', nie:url(?song)
-                )
-            )
-            FILTER (
-                NOT EXISTS {
-                    ?song a nmm:Video
-                } &&
-                NOT EXISTS {
-                    ?song a nmm:Playlist
-                }
-            )
-        }
-    ORDER BY ?artist ?album nmm:setNumber(nmm:musicAlbumDisc(?song)) nmm:trackNumber(?song)
+        tracker:id (?song) AS ?id
+        ?url
+        nie:title(?song) AS ?title
+        nmm:artistName (nmm:performer(?song)) AS ?artist
+        nie:title (nmm:musicAlbum (?song)) AS ?album
+        nfo:duration (?song) AS ?duration
+        IF (BOUND (?tag), 'b', '') AS ?lyrics
+    {
+        %(where_clause)s
+        ?song a nmm:MusicPiece ;
+            nmm:musicAlbumDisc ?disc ;
+            nmm:musicAlbum ?album ;
+            nmm:performer ?performer ;
+            nie:url ?url .
+        ?album nmm:albumArtist ?albumArtist ;
+            nie:title ?title .
+        OPTIONAL { ?song nao:hasTag ?tag .
+                   FILTER (?tag = nao:predefined-tag-favorite) } .
+        FILTER (STRSTARTS (?url, '%(music_dir)s/'))
+    }
+    ORDER BY ?artist ?album nmm:setNumber(?disc) nmm:trackNumber(?song)
     '''.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
-            'music_dir': Query.MUSIC_URI,
-            'download_dir': Query.DOWNLOAD_URI
+            'music_dir': Query.MUSIC_URI
         }
 
         return query
@@ -485,28 +234,17 @@ class Query():
         nfo:entryCounter(?playlist) AS childcount
         {
             %(where_clause)s
+            ?playlist a nmm:Playlist .
             OPTIONAL {
-                ?playlist a nfo:FileDataObject .
-                FILTER (
-                    EXISTS {
-                        ?playlist tracker:available 'true'
-                        FILTER (
-                            tracker:uri-is-descendant(
-                                '%(music_dir)s', nie:url(?playlist)
-                            ) ||
-                            tracker:uri-is-descendant(
-                                '%(download_dir)s', nie:url(?playlist)
-                            )
-                        )
-                    }
-                )
+                ?playlist nie:url ?url ;
+                          tracker:available true .
+            FILTER (STRSTARTS (?url, '%(music_dir)s/'))
             }
         }
-    ORDER BY fn:lower-case(?title)
+    ORDER BY LCASE(?title)
     '''.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
-            'music_dir': Query.MUSIC_URI,
-            'download_dir': Query.DOWNLOAD_URI
+            'music_dir': Query.MUSIC_URI
         }
 
         return query
@@ -527,29 +265,10 @@ class Query():
         ?song a nmm:MusicPiece ;
               a nfo:FileDataObject ;
               nmm:musicAlbum ?album .
-        OPTIONAL {
-            ?song nao:hasTag ?tag .
-            FILTER( ?tag = nao:predefined-tag-favorite )
-        }
-        FILTER (
-            tracker:id(?album) = %(album_id)s
-        )
-        FILTER (
-            tracker:uri-is-descendant(
-                '%(music_dir)s', nie:url(?song)
-            ) ||
-            tracker:uri-is-descendant(
-                '%(download_dir)s', nie:url(?song)
-            )
-        )
-        FILTER (
-            NOT EXISTS {
-                ?song a nmm:Video
-            } &&
-            NOT EXISTS {
-                ?song a nmm:Playlist
-            }
-        )
+        OPTIONAL { ?song nao:hasTag ?tag .
+                   FILTER( ?tag = nao:predefined-tag-favorite ) } .
+        FILTER (tracker:id(?album) = %(album_id)s &&
+                (STRSTARTS(nie:url(?song), '%(music_dir)s/')))
     }
     ORDER BY
          nmm:setNumber(nmm:musicAlbumDisc(?song))
@@ -557,8 +276,7 @@ class Query():
          tracker:added(?song)
     '''.replace('\n', ' ').strip() % {
             'album_id': album_id,
-            'music_dir': Query.MUSIC_URI,
-            'download_dir': Query.DOWNLOAD_URI
+            'music_dir': Query.MUSIC_URI
         }
 
         return query
@@ -605,8 +323,7 @@ class Query():
     '''.replace('\n', ' ').strip() % {
             'playlist_id': playlist_id,
             'filter_clause': filter_clause or 'tracker:id(?playlist) = ' + playlist_id,
-            'music_dir': Query.MUSIC_URI,
-            'download_dir': Query.DOWNLOAD_URI
+            'music_dir': Query.MUSIC_URI
         }
 
         return query
@@ -635,8 +352,7 @@ class Query():
     }
     """.replace("\n", " ").strip() % {
             'album_id': album_id,
-            'music_dir': Query.MUSIC_URI,
-            'download_dir': Query.DOWNLOAD_URI
+            'music_dir': Query.MUSIC_URI
         }
         return query
 
@@ -665,9 +381,6 @@ class Query():
         FILTER (
             tracker:uri-is-descendant(
                 '%(music_dir)s', nie:url(?song)
-            ) ||
-            tracker:uri-is-descendant(
-                '%(download_dir)s', nie:url(?song)
             )
         )
         FILTER (
@@ -681,8 +394,7 @@ class Query():
     }
     """.replace("\n", " ").strip() % {
             'song_id': song_id,
-            'music_dir': Query.MUSIC_URI,
-            'download_dir': Query.DOWNLOAD_URI
+            'music_dir': Query.MUSIC_URI
         }
         return query
 
@@ -1000,15 +712,11 @@ class Query():
           FILTER (
             tracker:uri-is-descendant(
               '%(music_dir)s', nie:url(?song)
-            ) ||
-            tracker:uri-is-descendant(
-              '%(download_dir)s', nie:url(?song)
             )
           )
         } ORDER BY DESC(?count) LIMIT 50
         """.replace('\n', ' ').strip() % {
-            'music_dir': Query.MUSIC_URI,
-            'download_dir': Query.DOWNLOAD_URI
+            'music_dir': Query.MUSIC_URI
         }
 
         return query
@@ -1025,15 +733,11 @@ class Query():
             FILTER (
             tracker:uri-is-descendant(
                 '%(music_dir)s', nie:url(?song)
-            ) ||
-            tracker:uri-is-descendant(
-                '%(download_dir)s', nie:url(?song)
             )
         )
         } ORDER BY nfo:fileLastAccessed(?song) LIMIT 50
         """.replace('\n', ' ').strip() % {
-            'music_dir': Query.MUSIC_URI,
-            'download_dir': Query.DOWNLOAD_URI
+            'music_dir': Query.MUSIC_URI
         }
 
         return query
@@ -1060,16 +764,12 @@ class Query():
                 FILTER (
                   tracker:uri-is-descendant(
                     '%(music_dir)s', nie:url(?song)
-                  ) ||
-                  tracker:uri-is-descendant(
-                    '%(download_dir)s', nie:url(?song)
                   )
                 )
             } ORDER BY DESC(?last_played) LIMIT 50
             """.replace('\n', ' ').strip() % {
                 'compare_date': compare_date,
-                'music_dir': Query.MUSIC_URI,
-                'download_dir': Query.DOWNLOAD_URI
+                'music_dir': Query.MUSIC_URI
             }
 
             return query
@@ -1096,16 +796,12 @@ class Query():
             FILTER (
               tracker:uri-is-descendant(
                 '%(music_dir)s', nie:url(?song)
-              ) ||
-              tracker:uri-is-descendant(
-                '%(download_dir)s', nie:url(?song)
               )
             )
         } ORDER BY DESC(?added) LIMIT 50
         """.replace('\n', ' ').strip() % {
             'compare_date': compare_date,
-            'music_dir': Query.MUSIC_URI,
-            'download_dir': Query.DOWNLOAD_URI
+            'music_dir': Query.MUSIC_URI
         }
 
         return query
@@ -1122,15 +818,11 @@ class Query():
         FILTER (
           tracker:uri-is-descendant(
             '%(music_dir)s', nie:url(?song)
-          ) ||
-          tracker:uri-is-descendant(
-            '%(download_dir)s', nie:url(?song)
           )
         )
     } ORDER BY DESC(tracker:added(?song))
     """.replace('\n', ' ').strip() % {
             'music_dir': Query.MUSIC_URI,
-            'download_dir': Query.DOWNLOAD_URI
         }
 
         return query
@@ -1139,17 +831,23 @@ class Query():
     # TODO: make those queries actually return something
     @staticmethod
     def get_albums_with_any_match(name):
-        name = Tracker.sparql_escape_string(GLib.utf8_casefold(name, -1))
+        name = Tracker.sparql_escape_string(GLib.utf8_normalize(GLib.utf8_casefold(name, -1), -1, GLib.NormalizeMode.NFKD))
         query = '''
             {
                 SELECT DISTINCT
                     nmm:musicAlbum(?song) AS album
-                WHERE {
+                {
                     ?song a nmm:MusicPiece .
+                    BIND(tracker:normalize(nie:title(nmm:musicAlbum(?song)), 'nfkd') AS ?match1) .
+                    BIND(tracker:normalize(nmm:artistName(nmm:performer(?song)), 'nfkd') AS ?match2) .
+                    BIND(tracker:normalize(nie:title(?song), 'nfkd') AS ?match3) .
                     FILTER (
-                        fn:contains(tracker:case-fold(nie:title(nmm:musicAlbum(?song))), "%(name)s") ||
-                        fn:contains(tracker:case-fold(nmm:artistName(nmm:performer(?song))), "%(name)s") ||
-                        fn:contains(tracker:case-fold(nie:title(?song)), "%(name)s")
+                        CONTAINS(tracker:case-fold(tracker:unaccent(?match1)), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(?match1), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(tracker:unaccent(?match2)), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(?match2), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(tracker:unaccent(?match3)), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(?match3), "%(name)s")
                     )
                 }
             }
@@ -1159,72 +857,44 @@ class Query():
 
     @staticmethod
     def get_albums_with_artist_match(name):
-        name = Tracker.sparql_escape_string(GLib.utf8_casefold(name, -1))
-        query = '''
-            {
-                SELECT DISTINCT
-                    ?album
-                WHERE {
-                    ?album a nmm:MusicAlbum ;
-                        nmm:albumArtist ?artist .
-                    FILTER (
-                        fn:contains(tracker:case-fold(nmm:artistName(?artist)), "%(name)s")
-                    )
-                }
-            }
-            '''.replace('\n', ' ').strip() % {'name': name}
+        name = Tracker.sparql_escape_string(name)
+        query = '?performer fts:match "nmm:artistName: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
 
         return Query.albums(query)
 
     @staticmethod
     def get_albums_with_album_match(name):
-        name = Tracker.sparql_escape_string(GLib.utf8_casefold(name, -1))
-        query = '''
-            {
-                SELECT DISTINCT
-                    ?album
-                WHERE {
-                    ?album a nmm:MusicAlbum .
-                    FILTER (
-                        fn:contains(tracker:case-fold(nie:title(?album)), "%(name)s")
-                    )
-                }
-            }
-            '''.replace('\n', ' ').strip() % {'name': name}
+        name = Tracker.sparql_escape_string(name)
+        query = '?album fts:match "nie:title: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
 
         return Query.albums(query)
 
     @staticmethod
     def get_albums_with_track_match(name):
-        name = Tracker.sparql_escape_string(GLib.utf8_casefold(name, -1))
-        query = '''
-            {
-                SELECT DISTINCT
-                    nmm:musicAlbum(?song) AS album
-                WHERE {
-                    ?song a nmm:MusicPiece .
-                    FILTER (
-                        fn:contains(tracker:case-fold(nie:title(?song)), "%(name)s")
-                    )
-                }
-            }
-            '''.replace('\n', ' ').strip() % {'name': name}
+        name = Tracker.sparql_escape_string(name)
+        query = '?song fts:match "nie:title: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
 
         return Query.albums(query)
 
     @staticmethod
     def get_artists_with_any_match(name):
-        name = Tracker.sparql_escape_string(GLib.utf8_casefold(name, -1))
+        name = Tracker.sparql_escape_string(GLib.utf8_normalize(GLib.utf8_casefold(name, -1), -1, GLib.NormalizeMode.NFKD))
         query = '''
             {
                 SELECT DISTINCT
                     nmm:musicAlbum(?song) AS album
-                WHERE {
+                {
                     ?song a nmm:MusicPiece .
+                    BIND(tracker:normalize(nie:title(nmm:musicAlbum(?song)), 'nfkd') AS ?match1) .
+                    BIND(tracker:normalize(nmm:artistName(nmm:performer(?song)), 'nfkd') AS ?match2) .
+                    BIND(tracker:normalize(nie:title(?song), 'nfkd') AS ?match3) .
                     FILTER (
-                        fn:contains(tracker:case-fold(nie:title(nmm:musicAlbum(?song))), "%(name)s") ||
-                        fn:contains(tracker:case-fold(nmm:artistName(nmm:performer(?song))), "%(name)s") ||
-                        fn:contains(tracker:case-fold(nie:title(?song)), "%(name)s")
+                        CONTAINS(tracker:case-fold(tracker:unaccent(?match1)), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(?match1), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(tracker:unaccent(?match2)), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(?match2), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(tracker:unaccent(?match3)), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(?match3), "%(name)s")
                     )
                 }
             }
@@ -1234,72 +904,44 @@ class Query():
 
     @staticmethod
     def get_artists_with_artist_match(name):
-        name = Tracker.sparql_escape_string(GLib.utf8_casefold(name, -1))
-        query = '''
-            {
-                SELECT DISTINCT
-                    ?album
-                WHERE {
-                    ?album a nmm:MusicAlbum ;
-                        nmm:albumArtist ?artist .
-                    FILTER (
-                        fn:contains(tracker:case-fold(nmm:artistName(?artist)), "%(name)s")
-                    )
-                }
-            }
-            '''.replace('\n', ' ').strip() % {'name': name}
+        name = Tracker.sparql_escape_string(name)
+        query = '?performer fts:match "nmm:artistName: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
 
         return Query.artists(query)
 
     @staticmethod
     def get_artists_with_album_match(name):
-        name = Tracker.sparql_escape_string(GLib.utf8_casefold(name, -1))
-        query = '''
-            {
-                SELECT DISTINCT
-                    ?album
-                WHERE {
-                    ?album a nmm:MusicAlbum .
-                    FILTER (
-                        fn:contains(tracker:case-fold(nie:title(?album)), "%(name)s")
-                    )
-                }
-            }
-            '''.replace('\n', ' ').strip() % {'name': name}
+        name = Tracker.sparql_escape_string(name)
+        query = '?album fts:match "nie:title: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
 
         return Query.artists(query)
 
     @staticmethod
     def get_artists_with_track_match(name):
-        name = Tracker.sparql_escape_string(GLib.utf8_casefold(name, -1))
-        query = '''
-            {
-                SELECT DISTINCT
-                    nmm:musicAlbum(?song) AS album
-                WHERE {
-                    ?song a nmm:MusicPiece .
-                    FILTER (
-                        fn:contains(tracker:case-fold(nie:title(?song)), "%(name)s")
-                    )
-                }
-            }
-            '''.replace('\n', ' ').strip() % {'name': name}
+        name = Tracker.sparql_escape_string(name)
+        query = '?song fts:match "nie:title: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
 
         return Query.artists(query)
 
     @staticmethod
     def get_songs_with_any_match(name):
-        name = Tracker.sparql_escape_string(GLib.utf8_casefold(name, -1))
+        name = Tracker.sparql_escape_string(GLib.utf8_normalize(GLib.utf8_casefold(name, -1), -1, GLib.NormalizeMode.NFKD))
         query = '''
             {
                 SELECT DISTINCT
                     ?song
                 WHERE {
                     ?song a nmm:MusicPiece .
+                    BIND(tracker:normalize(nie:title(nmm:musicAlbum(?song)), 'nfkd') AS ?match1) .
+                    BIND(tracker:normalize(nmm:artistName(nmm:performer(?song)), 'nfkd') AS ?match2) .
+                    BIND(tracker:normalize(nie:title(?song), 'nfkd') AS ?match3) .
                     FILTER (
-                        fn:contains(tracker:case-fold(nie:title(?song)), "%(name)s") ||
-                        fn:contains(tracker:case-fold(nmm:artistName(nmm:performer(?song))), "%(name)s") ||
-                        fn:contains(tracker:case-fold(nie:title(nmm:musicAlbum(?song))), "%(name)s")
+                        CONTAINS(tracker:case-fold(tracker:unaccent(?match1)), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(?match1), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(tracker:unaccent(?match2)), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(?match2), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(tracker:unaccent(?match3)), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(?match3), "%(name)s")
                     )
                 }
             }
@@ -1309,55 +951,22 @@ class Query():
 
     @staticmethod
     def get_songs_with_artist_match(name):
-        name = Tracker.sparql_escape_string(GLib.utf8_casefold(name, -1))
-        query = '''
-            {
-                SELECT DISTINCT
-                    ?song
-                WHERE {
-                    ?song a nmm:MusicPiece .
-                    FILTER (
-                        fn:contains(tracker:case-fold(nmm:artistName(nmm:performer(?song))), "%(name)s")
-                    )
-                }
-            }
-            '''.replace('\n', ' ').strip() % {'name': name}
+        name = Tracker.sparql_escape_string(name)
+        query = '?performer fts:match "nmm:artistName: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
 
         return Query.songs(query)
 
     @staticmethod
     def get_songs_with_album_match(name):
-        name = Tracker.sparql_escape_string(GLib.utf8_casefold(name, -1))
-        query = '''
-            {
-                SELECT DISTINCT
-                    ?song
-                WHERE {
-                    ?song a nmm:MusicPiece .
-                    FILTER (
-                        fn:contains(tracker:case-fold(nie:title(nmm:musicAlbum(?song))), "%(name)s")
-                    )
-                }
-            }
-            '''.replace('\n', ' ').strip() % {'name': name}
+        name = Tracker.sparql_escape_string(name)
+        query = '?album fts:match "nie:title: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
 
         return Query.songs(query)
 
     @staticmethod
     def get_songs_with_track_match(name):
-        name = Tracker.sparql_escape_string(GLib.utf8_casefold(name, -1))
-        query = '''
-            {
-                SELECT DISTINCT
-                    ?song
-                WHERE {
-                    ?song a nmm:MusicPiece .
-                    FILTER (
-                        fn:contains(tracker:case-fold(nie:title(?song)), "%(name)s")
-                    )
-                }
-            }
-            '''.replace('\n', ' ').strip() % {'name': name}
+        name = Tracker.sparql_escape_string(name)
+        query = '?song fts:match "nie:title: %(name)s*" . '.replace('\n', ' ').strip() % {'name': name}
 
         return Query.songs(query)
 
