@@ -92,7 +92,6 @@ class ViewContainer(Gtk.Stack):
             shadow_type=Gtk.ShadowType.NONE
         )
         self.view.set_view_type(view_type)
-        self.view.set_model(self.model)
         self._box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         self._box.pack_start(self.view, True, True, 0)
         if use_sidebar:
@@ -209,16 +208,10 @@ class ViewContainer(Gtk.Stack):
         self.window.notification.set_timeout(0)
         if not item:
             if remaining == 0:
+                self.view.set_model(self.model)
                 self.window.notification.dismiss()
                 self.view.show()
             return
-        # Make sure the item with this ID is not added
-        _iter = self.model.get_iter_first()
-        while _iter:
-            item_id = self.model.get_value(_iter, 0)
-            if str(item_id) == item.get_id():
-                return
-            _iter = self.model.iter_next(_iter)
 
         self._offset += 1
         artist = item.get_string(Grl.METADATA_KEY_ARTIST)\
@@ -227,22 +220,19 @@ class ViewContainer(Gtk.Stack):
         title = albumArtCache.get_media_title(item)
         # item.set_title(title)
 
-        def add_new_item():
-            _iter = self.model.append(None)
-            self.model.set(_iter,
-                           [0, 1, 2, 3, 4, 5, 7, 9],
-                           [str(item.get_id()), '', title,
-                            artist, self._loadingIcon, item,
-                            0, False])
-            self.cache.lookup(item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
-                              _iter, artist, title)
-        GLib.idle_add(add_new_item)
+        _iter = self.model.append(None)
+        self.model.set(_iter,
+                       [0, 1, 2, 3, 4, 5, 7, 9],
+                       [str(item.get_id()), '', title,
+                        artist, self._loadingIcon, item,
+                        0, False])
+        self.cache.lookup(item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
+                          _iter, artist, title)
 
     @log
     def _on_lookup_ready(self, icon, path, _iter):
         if icon:
             self.model.set_value(_iter, 4, icon)
-            self.view.queue_draw()
 
     @log
     def _add_list_renderers(self):
@@ -471,13 +461,13 @@ class Songs(ViewContainer):
         if self.model.get_value(currentIter, 8) != self.errorIconName:
             self.iter_to_clean = currentIter.copy()
 
-        self.view.queue_draw()
         return False
 
     def _add_item(self, source, param, item, remaining=0, data=None):
         self.window.notification.set_timeout(0)
         if not item:
             if remaining == 0:
+                self.view.set_model(self.model)
                 self.window.notification.dismiss()
                 self.view.show()
             return
@@ -734,6 +724,7 @@ class Artists (ViewContainer):
         self.window.notification.set_timeout(0)
         if item is None:
             if remaining == 0:
+                self.view.set_model(self.model)
                 self.window.notification.dismiss()
                 self.view.show()
             return
@@ -1033,7 +1024,6 @@ class Playlist(ViewContainer):
             self.iter_to_clean = currentIter.copy()
             self.iter_to_clean_model = self.model
 
-        self.view.queue_draw()
         return False
 
     @log
@@ -1142,7 +1132,7 @@ class Playlist(ViewContainer):
 
         # if the active queue has been set by this playlist,
         # use it as model, otherwise build the liststore
-        self.model = self.view.get_model()
+        self.view.set_model(None)
         self.model.clear()
         self.songs_count = 0
         GLib.idle_add(grilo.populate_playlist_songs, playlist, self._add_item)
@@ -1156,6 +1146,8 @@ class Playlist(ViewContainer):
     @log
     def _add_item(self, source, param, item, remaining=0, data=None):
         self._add_item_to_model(item, self.model)
+        if remaining == 0:
+            self.view.set_model(self.model)
 
     @log
     def _add_item_to_model(self, item, model):
@@ -1553,6 +1545,7 @@ class Search(ViewContainer):
 
         if self.model.iter_n_children(self.head_iters[group]) == 1:
             path = self.model.get_path(self.head_iters[group])
+            path = self.filter_model.convert_child_path_to_path(path)
             self.view.get_generic_view().expand_row(path, False)
 
     @log
